class IndexedDBHelper {
    constructor(dbName, dbVersion) {
        this.dbName = dbName;
        this.dbVersion = dbVersion;
        this.db = null;
    }

    /**
     * 打开数据库
     */
    openDatabase(upgradeCallback) {
        return new Promise((resolve, reject) => {
            const request = indexedDB.open(this.dbName, this.dbVersion);

            request.onsuccess = (event) => {
                this.db = event.target.result;
                console.log("Database opened successfully");
                resolve(this.db);
            };

            request.onerror = (event) => {
                console.error("Database error:", event.target.error);
                reject(event.target.error);
            };

            request.onupgradeneeded = (event) => {
                console.log("Database upgrade needed");
                const db = event.target.result;
                if (typeof upgradeCallback === "function") {
                    upgradeCallback(db, event);
                }
            };
        });
    }

    deleteDatabase() {
      return new Promise((resolve, reject) => {
        const request = indexedDB.deleteDatabase('gycharmtiktokdb'); // 'gycharmtiktokdb' 是数据库的名字

        request.onsuccess = function(event) {
          console.log("数据库删除成功");
          resolve(event); // 成功时，解析 Promise
        };

        request.onerror = function(event) {
          console.error("数据库删除失败", event.target.error);
          reject(event.target.error); // 失败时，拒绝 Promise
        };

        request.onblocked = function(event) {
          console.warn("数据库删除被阻塞");
          reject(event.target.error); // 阻塞时拒绝 Promise
        };
      });
    }


    /**
     * 添加数据
     */
    addData(storeName, data) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction([storeName], "readwrite");
            const store = transaction.objectStore(storeName);
            const request = store.add(data);

            request.onsuccess = () => {
                //console.log("Data added successfully");
                //console.log(data);
                resolve(request.result);
            };

            request.onerror = (event) => {
                console.error("Error adding data:", event.target.error);
                reject(event.target.error);
            };
        });
    }

    /**
     * 获取数据
     */
    getData(storeName, key) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction([storeName], "readonly");
            const store = transaction.objectStore(storeName);
            const request = store.get(key);

            request.onsuccess = () => {
                if (request.result === undefined) {
                    console.log(`No data found for key: ${key}`);
                    resolve(null); // 返回 null 更友好
                } else {
                    console.log("Data retrieved successfully");
                    resolve(request.result);
                }
            };

            request.onerror = (event) => {
                console.error("Error retrieving data:", event.target.error);
                reject(event.target.error);
            };
        });
    }

    /**
     * 获取所有数据
     */
    getAllData(storeName) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction([storeName], "readonly");
            const store = transaction.objectStore(storeName);
            const request = store.getAll();

            request.onsuccess = () => {
                console.log("All data retrieved successfully");
                resolve(request.result);
            };

            request.onerror = (event) => {
                console.error("Error retrieving all data:", event.target.error);
                reject(event.target.error);
            };
        });
    }

    /**
     * 更新数据
     */
    updateData(storeName, data) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction([storeName], "readwrite");
            const store = transaction.objectStore(storeName);
            const request = store.put(data);

            request.onsuccess = () => {
                console.log("Data updated successfully");
                resolve(request.result);
            };

            request.onerror = (event) => {
                console.error("Error updating data:", event.target.error);
                reject(event.target.error);
            };
        });
    }

    updateDataById(storeName, id, updatedFields){
        return new Promise((resolve, reject) =>{
            const transaction = this.db.transaction([storeName], "readwrite");
            const store = transaction.objectStore(storeName);

            //获取当前id存储对象
            const getRequest = store.get(id);

            getRequest.onsuccess = (event) => {
                const record = event.target.result;
                if(record){
                    //合并更细字段
                    const updatedRecord = { ...record, ...updatedFields };
                    // 更新记录
                    const updateRequest = store.put(updatedRecord);
                    //
                    updateRequest.onsuccess = () => {
                        console.log("Record updated successfully:", updatedRecord);
                        resolve(updatedRecord);
                    };

                    updateRequest.onerror = (error) => {
                        console.error("Error updating record:", error);
                        reject(error);
                    };
                }else{
                    //
                    console.error(`Record with id ${id} not found.`);
                    reject(new Error(`Record with id ${id} not found.`));
                    //
                }
            }

            getRequest.onerror = (event) => {
                console.error("Error fetching record:", error);
                reject(error);
            }
        });
    }

    /**
     * 删除数据
     */
    deleteData(storeName, key) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction([storeName], "readwrite");
            const store = transaction.objectStore(storeName);
            const request = store.delete(key);

            request.onsuccess = () => {
                console.log("Data deleted successfully");
                resolve(request.result);
            };

            request.onerror = (event) => {
                console.error("Error deleting data:", event.target.error);
                reject(event.target.error);
            };
        });
    }

    /**
     * 清空对象存储
     */
    clearStore(storeName) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction([storeName], "readwrite");
            const store = transaction.objectStore(storeName);
            const request = store.clear();

            request.onsuccess = () => {
                console.log("Object store cleared successfully");
                resolve(request.result);
            };

            request.onerror = (event) => {
                console.error("Error clearing object store:", event.target.error);
                reject(event.target.error);
            };
        });
    }

    /**
     * 清空已采集 针对products
     */
    clearCustomsHas(){
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['customers'], "readwrite");
            const store = transaction.objectStore('customers');
            const index = store.index('_is_request_index');  // 获取索引

            const cursorRequest = index.openCursor(IDBKeyRange.only(2));

            cursorRequest.onsuccess = (event) => {
                const cursor = event.target.result;
                //
                if (cursor) {
                    const record_data = cursor.value;
                    if (record_data['_is_request'] == 2) {
                        // 如果找到符合条件的记录，删除它
                        console.log('cursor.primaryKey:'+cursor.primaryKey); // 获取主键
                        store.delete(cursor.primaryKey);
                        console.log(`Deleted item with primaryKey: ${cursor.primaryKey}`);
                    }
                    cursor.continue(); // 继续遍历
                } else {
                    console.log("All matching records deleted.");
                    resolve();
                }
                //
            };

            cursorRequest.onerror = (event) => {
                console.error("Error during cursor iteration:", event.target.error);
                reject(event.target.error);
            };
        });
    }

    /**
     * 查询符合采集条件的产品
     */
    findNeedSpiderProduct(storeName){
        return new Promise((resolve, reject) => {
            let transaction = this.db.transaction([storeName], 'readonly');
            let store = transaction.objectStore(storeName);
            let index = store.index('_is_request_index');
             // 使用索引游标按倒序（prev）遍历 ,目前用升序
            let request = index.openCursor(1); //查询 _is_request = 1 且倒序
            request.onsuccess = (event) => {
                let cursor = event.target.result;
                if(cursor){
                    console.log("findNeedSpiderProduct data:"+ cursor.value);
                    let item =  cursor.value;
                    if(item['_s_time'] < (Date.now()-10000)){ //60000
                        resolve(cursor.value);
                    }
                }
            };
            request.onerror = (event) => {
                console.error("findNeedSpiderProduct Error:", event.target.error);
                reject('error');
            }
            //
            let request2 = index.openCursor(0); //查询 _is_request = 0 且倒序
            request2.onsuccess = (event) => {
                let cursor = event.target.result;
                if(cursor){
                    console.log("findNeedSpiderProduct data:"+ cursor.value);
                    let item =  cursor.value;
                    resolve(item);
                }else{
                    resolve(null);
                }
            };
            request2.onerror = (event) => {
                console.error("findNeedSpiderProduct Error:", event.target.error);
                reject('error');
            }
            //
        });
    }

    /**
     * 查询个人需要采集的任务
     */
    findGerenNeedSpide() {
      return new Promise((resolve, reject) => {
          let transaction = this.db.transaction(['customers'], 'readonly');
          let store = transaction.objectStore('customers');
          let index = store.index('_is_request_index');
          let keyRange = IDBKeyRange.only(0); // _is_request_index = 0 目前未用该字段

          index.openCursor(keyRange).onsuccess = (event) => {
              let cursor = event.target.result;
              if(cursor){
                  console.log("findGerenNeedSpide data:"+ cursor.value);
                  let item =  cursor.value;
                  resolve(item);
              }else{
                  resolve(null);
              }
          };

          index.openCursor(keyRange).onerror = (event) => {
            reject("游标查询失败: " + event.target.error);
          };

      });
    }

    /**
     * 查询个人company_page_link
     */
    findGetcompany_page_linkNeedSpide() {
      return new Promise((resolve, reject) => {
          let transaction = this.db.transaction(['customers'], 'readonly');
          let store = transaction.objectStore('customers');
          let index = store.index('_shangjia_company_is_request_index');
          let keyRange = IDBKeyRange.only(0); // _shangjia_company_is_request_index = 0 目前未用该字段

          index.openCursor(keyRange).onsuccess = (event) => {
              let cursor = event.target.result;
              if(cursor){
                  console.log("findGetcompany_page_linkNeedSpide data:"+ cursor.value);
                  let item =  cursor.value;
                  resolve(item);
              }else{
                  resolve(null);
              }
          };

          index.openCursor(keyRange).onerror = (event) => {
            reject("游标查询失败: " + event.target.error);
          };

      });
    }

    /**
     * 查询个人company_info
     * @returns {Promise<unknown>}
     */
    findGetcompany_page_InfoNeedSpide() {
      return new Promise((resolve, reject) => {
          let transaction = this.db.transaction(['customers'], 'readonly');
          let store = transaction.objectStore('customers');
          let index = store.index('_shangjia_company_is_request_index');
          let keyRange = IDBKeyRange.only(2); // _shangjia_company_is_request_index = 0 目前未用该字段

          index.openCursor(keyRange).onsuccess = (event) => {
              let cursor = event.target.result;
              if(cursor){
                  console.log("findGetcompany_page_InfoNeedSpide data:"+ cursor.value);
                  let item =  cursor.value;
                  if(item['_awel_is_request'] ==0){
                      resolve(item);
                  }
                  cursor.continue(); // 继续查找
              }else{
                  resolve(null);
              }
          };

          index.openCursor(keyRange).onerror = (event) => {
            reject("游标查询失败: " + event.target.error);
          };

      });
    }

    //更新_shangjia_followed
    updateFollowed(id,_shangjia_followed=0) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['customers'], 'readwrite');
            const store = transaction.objectStore('customers');
            // 获取指定 id 的记录
            const getRequest = store.get(id);

            getRequest.onsuccess = (event) => {
                const record = event.target.result;
                if (record) {
                    record['basicinfo']['_shangjia_followed'] = _shangjia_followed;
                    const updateRequest = store.put(record); // 写入更新

                    updateRequest.onsuccess = () => resolve(record.status);
                    updateRequest.onerror = (event) => reject(event.target.error);
                } else {
                    reject(new Error(`Record with id ${id} not found`));
                }
            };

            getRequest.onerror = (event) => reject(event.target.error);
        });
    }

    //获取任务列表
    findTaskList(){
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['huoke_task'], "readonly");
            const store = transaction.objectStore('huoke_task');

            const request = store.openCursor(null, "prev"); // "prev" 表示降序

            const results = [];
            request.onsuccess = (event) => {
                const cursor = event.target.result;
                if (cursor) {
                    results.push(cursor.value);
                    cursor.continue();
                } else {
                    console.log("All data retrieved and sorted by id descending");
                    resolve(results);
                }
            };

            request.onerror = (event) => {
                console.error("Error retrieving all data:", event.target.error);
                reject(event.target.error);
            };
        });
    }

    //获取需要采集的task_item
    findNeedTaskItem(task_id){
        return new Promise((resolve, reject) => {
            let transaction = this.db.transaction(['huoke_task_item'], 'readonly');
            let store = transaction.objectStore('huoke_task_item');
            let index = store.index('_task_id_index');
            let keyRange = IDBKeyRange.only(task_id); // _is_request_index = 0
            const cursorRequest = index.openCursor(keyRange, "next"); // 按 id 升序
            let result = null;

            cursorRequest.onsuccess = (event) => {
                const cursor = event.target.result;
                if (cursor) {
                    const record = cursor.value;
                    if (record._is_request === 0) {
                        result = record;
                        resolve(result); // 找到第一条符合条件的数据后立即返回
                        return;
                    }
                    cursor.continue(); // 继续查找
                } else {
                    resolve(null); // 没有找到符合条件的记录
                }
            };
            cursorRequest.onerror = (event) => reject(event.target.error);
      });
    }

    //
    getTaskitemPaginatedData(req_args, page, pageSize = 10){
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['huoke_task_item'], "readonly");
            const store = transaction.objectStore('huoke_task_item');
            //
            let historyRecoirdId = req_args['historyRecoirdId'];
            console.log('historyRecoirdId');
            console.log(historyRecoirdId);
            let _temp_arr = historyRecoirdId.split('_');
            let _task_id = _temp_arr[_temp_arr.length-1];
            _task_id = parseInt(_task_id);
            //
            let query_args = {};
            if(req_args.hasOwnProperty('query_args')){
                query_args = req_args['query_args'];
            }
            //
            let index = store.index('_task_id_index');
            let keyRange = IDBKeyRange.only(_task_id); // _is_request_index = 0
            const cursorRequest = index.openCursor(keyRange, "next"); // 按 id 升序
            /////
            let taskitems = [];
            cursorRequest.onsuccess = (event) => {
                const cursor = event.target.result;
                if (cursor) {
                    let taskitem_value = cursor.value;
                     //判断是否是没有筛选项
                    let is_query_args = true;
                    if (Object.keys(query_args).length > 0) {
                        if(query_args['followers_s'] ==0 && query_args['followers_e'] ==0 &&
                            query_args['following_s'] ==0 && query_args['following_e'] ==0 &&
                            query_args['posts_s'] ==0 && query_args['posts_s'] ==0 &&
                            query_args['is_business'] == "All Accounts" && query_args['is_private'] == 'All Accounts' &&
                            query_args['is_verified'] == "All Accounts" && query_args['bio_include'] == '' && query_args['bio_exclude'] == ''
                        ){
                            is_query_args = false;
                        }
                    }else{
                        is_query_args = false;
                    }
                    if (is_query_args) {
                        if(taskitem_value['_is_request'] == 1){
                            let followers = taskitem_value['other_data']['followers'];
                            let following = taskitem_value['other_data']['following'];
                            let posts = taskitem_value['other_data']['posts'];
                            let is_private = taskitem_value['other_data']['is_private'];
                            let is_verified = taskitem_value['other_data']['is_verified'];
                            let is_business = taskitem_value['other_data']['is_business'];
                            let bio = taskitem_value['other_data']['bio'];
                            //
                            let is_cond = true;
                            if(followers < query_args['followers_s']){
                                is_cond = false;
                            }
                            if(query_args['followers_e'] > 0 && followers > query_args['followers_e']){
                                is_cond = false;
                            }
                            if(following < query_args['following_s']){
                                is_cond = false;
                            }
                            if(query_args['following_e'] > 0 && following > query_args['following_e']){
                                is_cond = false;
                            }
                            if(posts < query_args['posts_s']){
                                is_cond = false;
                            }
                            if(query_args['posts_e'] > 0 && posts > query_args['posts_e']){
                                is_cond = false;
                            }
                            //
                            let query_args_is_private = true;
                            if(query_args['is_private'] == 'Public Accounts'){
                                query_args_is_private = false;
                            }
                            if(query_args['is_private'] != 'All Accounts'){
                                if(is_private!=query_args_is_private){
                                    is_cond = false;
                                }
                            }
                            //
                            let query_args_is_verified = true;
                            if(query_args['is_verified'] == 'Non-Verified Accounts'){
                                query_args_is_verified = false;
                            }
                            if(query_args['is_verified'] != 'All Accounts'){
                                if(is_verified!=query_args_is_verified){
                                    is_cond = false;
                                }
                            }
                            //
                            let query_args_is_business = true;
                            if(query_args['is_business'] == 'Non-Business Accounts'){
                                query_args_is_business = false;
                            }
                            if(query_args['is_business'] != 'All Accounts'){
                                if(is_business!=query_args_is_business){
                                    is_cond = false;
                                }
                            }
                            //
                            let query_args_bio_include = query_args['bio_include'];
                            if(bio && bio.length > 0){
                                bio = bio.toLowerCase();
                                if(query_args_bio_include.length > 0){
                                    let is_include = false;
                                    let query_args_bio_include_arr = query_args_bio_include.split('#');
                                    for(let _bio_word of query_args_bio_include_arr){
                                        _bio_word = _bio_word.trim();
                                        _bio_word = _bio_word.toLowerCase();
                                        if(bio.includes(_bio_word)){
                                            is_include =true;
                                            break;
                                        }
                                    }
                                    if(!is_include){
                                        is_cond = false;
                                    }
                                }
                            }
                            //
                            let query_args_bio_exclude = query_args['bio_exclude'];
                            if(bio && bio.length > 0){
                                bio = bio.toLowerCase();
                                if(query_args_bio_exclude.length > 0){
                                    let is_exclude = true;//表示不包含
                                    let query_args_bio_exclude_arr = query_args_bio_exclude.split('#');
                                    for(let _bio_word of query_args_bio_exclude_arr){
                                        _bio_word = _bio_word.trim();
                                        _bio_word = _bio_word.toLowerCase();
                                        if(bio.includes(_bio_word)){
                                            is_exclude = false;
                                            break;
                                        }
                                    }
                                    if(!is_exclude){
                                        is_cond = false;
                                    }
                                }
                            }
                            //
                            if(query_args_bio_include.length >0 || query_args_bio_exclude.length >0){
                                if(!bio || bio.length == 0){
                                    is_cond = false;
                                }
                            }
                            //
                            if(is_cond){
                                taskitems.push(cursor.value);
                            }
                            //
                        }
                        //
                    }else{
                        taskitems.push(taskitem_value);
                    }
                    //
                    cursor.continue(); // 继续读取下一条
                } else {
                    // 所有记录已经读取完成，开始分页
                    const start = (page - 1) * pageSize;
                    const end = start + pageSize;
                    const paginatedData = taskitems.slice(start, end);
                    resolve({
                        'total':taskitems.length,
                        'data':paginatedData
                    });
                }
            };
            cursorRequest.onerror = (event) => reject(event.target.error);
            //
        });
    }

    getTaskitems(req_args, export_type=''){
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['huoke_task_item'], "readonly");
            const store = transaction.objectStore('huoke_task_item');
            //
            let historyRecoirdId = req_args['historyRecoirdId'];
            console.log('historyRecoirdId');
            console.log(historyRecoirdId);
            let _temp_arr = historyRecoirdId.split('_');
            let _task_id = _temp_arr[_temp_arr.length-1];
            _task_id = parseInt(_task_id);
            //
            let query_args = {};
            if(req_args.hasOwnProperty('query_args')){
                query_args = req_args['query_args'];
            }
            //
            let index = store.index('_task_id_index');
            let keyRange = IDBKeyRange.only(_task_id); // _is_request_index = 0
            const cursorRequest = index.openCursor(keyRange, "next"); // 按 id 升序
            /////
            let taskitems = [];
            cursorRequest.onsuccess = (event) => {
                const cursor = event.target.result;
                if (cursor) {
                    let taskitem_value = cursor.value;
                     //判断是否是没有筛选项
                    let is_query_args = true;
                    if (Object.keys(query_args).length > 0) {
                        if(query_args['followers_s'] ==0 && query_args['followers_e'] ==0 &&
                            query_args['following_s'] ==0 && query_args['following_e'] ==0 &&
                            query_args['posts_s'] ==0 && query_args['posts_s'] ==0 &&
                            query_args['is_business'] == "All Accounts" && query_args['is_private'] == 'All Accounts' &&
                            query_args['is_verified'] == "All Accounts" && query_args['bio_include'] == '' && query_args['bio_exclude'] == ''
                        ){
                            is_query_args = false;
                        }
                    }else{
                        is_query_args = false;
                    }
                    if (is_query_args && export_type == '') {
                        if(taskitem_value['_is_request'] == 1){
                            let followers = taskitem_value['other_data']['followers'];
                            let following = taskitem_value['other_data']['following'];
                            let posts = taskitem_value['other_data']['posts'];
                            let is_private = taskitem_value['other_data']['is_private'];
                            let is_verified = taskitem_value['other_data']['is_verified'];
                            let is_business = taskitem_value['other_data']['is_business'];
                            let bio = taskitem_value['other_data']['bio'];
                            //
                            let is_cond = true;
                            if(followers < query_args['followers_s']){
                                is_cond = false;
                            }
                            if(query_args['followers_e'] > 0 && followers > query_args['followers_e']){
                                is_cond = false;
                            }
                            if(following < query_args['following_s']){
                                is_cond = false;
                            }
                            if(query_args['following_e'] > 0 && following > query_args['following_e']){
                                is_cond = false;
                            }
                            if(posts < query_args['posts_s']){
                                is_cond = false;
                            }
                            if(query_args['posts_e'] > 0 && posts > query_args['posts_e']){
                                is_cond = false;
                            }
                            //
                            let query_args_is_private = true;
                            if(query_args['is_private'] == 'Public Accounts'){
                                query_args_is_private = false;
                            }
                            if(query_args['is_private'] != 'All Accounts'){
                                if(is_private!=query_args_is_private){
                                    is_cond = false;
                                }
                            }
                            //
                            let query_args_is_verified = true;
                            if(query_args['is_verified'] == 'Non-Verified Accounts'){
                                query_args_is_verified = false;
                            }
                            if(query_args['is_verified'] != 'All Accounts'){
                                if(is_verified!=query_args_is_verified){
                                    is_cond = false;
                                }
                            }
                            //
                            let query_args_is_business = true;
                            if(query_args['is_business'] == 'Non-Business Accounts'){
                                query_args_is_business = false;
                            }
                            if(query_args['is_business'] != 'All Accounts'){
                                if(is_business!=query_args_is_business){
                                    is_cond = false;
                                }
                            }
                            //
                            let query_args_bio_include = query_args['bio_include'];
                            if(bio && bio.length > 0){
                                bio = bio.toLowerCase();
                                if(query_args_bio_include.length > 0){
                                    let is_include = false;
                                    let query_args_bio_include_arr = query_args_bio_include.split('#');
                                    for(let _bio_word of query_args_bio_include_arr){
                                        _bio_word = _bio_word.trim();
                                        _bio_word = _bio_word.toLowerCase();
                                        if(bio.includes(_bio_word)){
                                            is_include =true;
                                            break;
                                        }
                                    }
                                    if(!is_include){
                                        is_cond = false;
                                    }
                                }
                            }
                            //
                            let query_args_bio_exclude = query_args['bio_exclude'];
                            if(bio && bio.length > 0){
                                bio = bio.toLowerCase();
                                if(query_args_bio_exclude.length > 0){
                                    let is_exclude = true;//表示不包含
                                    let query_args_bio_exclude_arr = query_args_bio_exclude.split('#');
                                    for(let _bio_word of query_args_bio_exclude_arr){
                                        _bio_word = _bio_word.trim();
                                        _bio_word = _bio_word.toLowerCase();
                                        if(bio.includes(_bio_word)){
                                            is_exclude = false;
                                            break;
                                        }
                                    }
                                    if(!is_exclude){
                                        is_cond = false;
                                    }
                                }
                            }
                            //
                            if(query_args_bio_include.length >0 || query_args_bio_exclude.length >0){
                                if(!bio || bio.length == 0){
                                    is_cond = false;
                                }
                            }
                            //
                            if(is_cond){
                                taskitems.push(cursor.value);
                            }
                            //
                        }
                        //
                    }else{
                        if(export_type){
                            //
                            if(export_type == 'all'){
                                taskitems.push(taskitem_value);
                            }else if(export_type == 'email' && taskitem_value['_is_request'] == 1){
                                if(taskitem_value['other_data']['public_email']){
                                    taskitems.push(taskitem_value);
                                }
                            }else if(export_type == 'phone' && taskitem_value['_is_request'] == 1){
                                if(taskitem_value['other_data']['public_phone']){
                                    taskitems.push(taskitem_value);
                                }
                            }
                            //
                        }else{
                            taskitems.push(taskitem_value);
                        }
                    }
                    //
                    cursor.continue(); // 继续读取下一条
                } else {
                    resolve(taskitems);
                }
            };
            cursorRequest.onerror = (event) => reject(event.target.error);
            //
        });
    }

    //只有processing和pause两个状态更换
    updateTaskStatus(id) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['huoke_task'], 'readwrite');
            const store = transaction.objectStore('huoke_task');
            // 获取指定 id 的记录
            const getRequest = store.get(id);

            getRequest.onsuccess = (event) => {
                const record = event.target.result;
                if (record) {
                    if(record.status == 'processing' || record.status == 'pending'){
                        record.status = 'pause';
                    }else if(record.status == 'pause'){
                        record.status = 'processing';
                    }
                    const updateRequest = store.put(record); // 写入更新

                    updateRequest.onsuccess = () => resolve(record.status);
                    updateRequest.onerror = (event) => reject(event.target.error);
                } else {
                    reject(new Error(`Record with id ${id} not found`));
                }
            };

            getRequest.onerror = (event) => reject(event.target.error);
        });
    }

    //只有processing和pause两个状态更换
    updateTaskDstatus(id) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['huoke_task'], 'readwrite');
            const store = transaction.objectStore('huoke_task');
            // 获取指定 id 的记录
            const getRequest = store.get(id);

            getRequest.onsuccess = (event) => {
                const record = event.target.result;
                if (record) {
                    if(record.dstatus == 'processing' || record.dstatus == 'pending'){
                        record.dstatus = 'pause';
                    }else if(record.dstatus == 'pause'){
                        record.dstatus = 'processing';
                    }
                    const updateRequest = store.put(record); // 写入更新

                    updateRequest.onsuccess = () => resolve(record.dstatus);
                    updateRequest.onerror = (event) => reject(event.target.error);
                } else {
                    reject(new Error(`Record with id ${id} not found`));
                }
            };

            getRequest.onerror = (event) => reject(event.target.error);
        });
    }

    /*
    更新task
    task_item_has_count
    task_item_has_email_count
    task_item_has_phone_count
    task_item_has_website_count
     */
    async updateTaskSpideTongjiData(task_id) {
        try {
            let transaction = this.db.transaction(['huoke_task_item'], 'readonly');
            let store = transaction.objectStore('huoke_task_item');
            let index = store.index('_task_id_index');
            let keyRange = IDBKeyRange.only(task_id); // _is_request_index = 0
            const cursorRequest = index.openCursor(keyRange, "next"); // 按 id 升序
            //
            let task_item_has_count = 0;
            let task_item_has_email_count = 0;
            let task_item_has_phone_count = 0;
            let task_item_has_website_count = 0;

            cursorRequest.onsuccess = (event) => {
                const cursor = event.target.result;
                if (cursor) {
                    const record = cursor.value;
                    if (record._is_request === 1) {
                        task_item_has_count +=1;
                        if(record['other_data']['public_email']){
                            task_item_has_email_count+=1;
                        }
                        if(record['other_data']['public_phone']){
                            task_item_has_phone_count+=1;
                        }
                        if(record['other_data']['external_url']){
                            task_item_has_website_count+=1;
                        }
                    }
                    cursor.continue(); // 继续查找
                } else {
                    //更新
                    this.updateDataById('huoke_task', task_id, {
                        'task_item_has_count':task_item_has_count,
                        'task_item_has_email_count':task_item_has_email_count,
                        'task_item_has_phone_count':task_item_has_phone_count,
                        'task_item_has_website_count':task_item_has_website_count
                    })
                    //
                }
            };
            cursorRequest.onerror = (event) => reject(event.target.error);
        }catch (error){

        }
    }


    //获取taskItem总行数 已task_id查询
    getTaskItemCount(task_id){
        return new Promise((resolve, reject) => {
            let transaction = this.db.transaction(['huoke_task_item'], 'readonly');
            let store = transaction.objectStore('huoke_task_item');
            const index = store.index('_task_id_index'); // 使用 task_id 索引
            const range = IDBKeyRange.only(task_id); // 查询 task_id = 1
            let countRequest = index.count(range);

            countRequest.onsuccess = () => {
              resolve(countRequest.result);
            };

            countRequest.onerror = () => {
              reject(countRequest.error);
            };
        });
    }

    //标记task simple_status = 'complete';//表示采集粉丝、following链接完成
    updateTaskSimpleStatus(task_id) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['huoke_task'], 'readwrite');
            const store = transaction.objectStore('huoke_task');
            // 获取指定 id 的记录
            const getRequest = store.get(task_id);

            getRequest.onsuccess = (event) => {
                const record = event.target.result;
                if (record) {
                    record.simple_status = 'complete';
                    const updateRequest = store.put(record); // 写入更新

                    updateRequest.onsuccess = () => resolve(record.status);
                    updateRequest.onerror = (event) => reject(event.target.error);
                } else {
                    reject(new Error(`Record with id ${task_id} not found`));
                }
            };
            getRequest.onerror = (event) => reject(event.target.error);
        });
    }

    async markTaskComplete(task_id) {
       try{
           const transaction = this.db.transaction(["huoke_task"], "readwrite");
           const store = transaction.objectStore("huoke_task");
           const request = store.get(task_id);
           request.onsuccess = (event) => {
               //
               const record = event.target.result;
               if (record) {
                   record.status = 'complete';
                   store.put(record);
               }
               //
           };
       }catch (error){

       }
    }

    //task item 删除
    taskitemRemove(task_id){
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['huoke_task_item'], "readwrite");
            const store = transaction.objectStore('huoke_task_item');
            const index = store.index('_task_id_index');  // 获取索引

            const cursorRequest = index.openCursor(IDBKeyRange.only(task_id));

            cursorRequest.onsuccess = (event) => {
                const cursor = event.target.result;

                if (cursor) {
                    // 如果找到符合条件的记录，删除它
                    console.log('cursor.primaryKey:'+cursor.primaryKey); // 获取主键
                    store.delete(cursor.primaryKey);
                    console.log(`Deleted item with primaryKey: ${cursor.primaryKey}`);
                    cursor.continue();  // 继续遍历下一条记录
                } else {
                    console.log("All matching records deleted.");
                    resolve(true);
                }
            };

            cursorRequest.onerror = (event) => {
                console.error("Error during cursor iteration:", event.target.error);
                reject(event.target.error);
            };
        });
    }
    //dm task 删除
    async taskRemove(id){
        //
        let dmtaskitemdelete = await this.taskitemRemove(id);
        if(dmtaskitemdelete){
            this.deleteData('huoke_task', id);
        }
        return true;
    }

    //查询总记录数
    getRecordCount(storeName){
        return new Promise((resolve, reject) => {
            let transaction = this.db.transaction([storeName], 'readonly');
            let store = transaction.objectStore(storeName);
            //
            const request = store.count();

            request.onsuccess = (event) => {
                resolve(event.target.result); // 返回总记录数
            }

            request.onerror = (event) => {
                reject(event.target.error); // 捕获错误
            }
        });
    }

    getDataProcess(){
        return new Promise((resolve, reject) => {
            let transaction = this.db.transaction(['customers'], 'readonly');
            let store = transaction.objectStore('customers');
            let dataprocess = {
                'total':0,
                'has':0
            };
            const request = store.openCursor();
            request.onsuccess = (event) => {
                const cursor = event.target.result;
                if (cursor) {
                    const record_data = cursor.value;
                    dataprocess['total'] +=1;
                    if (record_data['_is_request'] == 2) {
                        dataprocess['has'] +=1;
                    }
                    cursor.continue(); // 继续遍历
                } else {
                    // 遍历结束
                    resolve(dataprocess);
                }
            };
            request.onerror = (event) => {
                reject(event.target.error);
            };
        });
    }

    //查询已完成的记录数
    //针对products
    getProductsHasRecordCount(){
        return new Promise((resolve, reject) => {
            let transaction = this.db.transaction(['products'], 'readonly');
            let store = transaction.objectStore('products');
            let count = 0;
            const request = store.openCursor();
            request.onsuccess = (event) => {
                const cursor = event.target.result;
                if (cursor) {
                    const record_data = cursor.value;
                    // 满足条件则计数
                    if (record_data['_is_request'] == 2) {
                        count++;
                    }
                    cursor.continue(); // 继续遍历
                } else {
                    // 遍历结束
                    resolve(count);
                }
            };
            request.onerror = (event) => {
                reject(event.target.error);
            };
        });
    }

    //dm
    //获取dm 模版列表
    getDmtemplatelist(){
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['dm_template'], "readonly");
            const store = transaction.objectStore('dm_template');

            const request = store.openCursor(null, "prev"); // "prev" 表示降序

            const results = [];
            request.onsuccess = (event) => {
                const cursor = event.target.result;
                if (cursor) {
                    results.push(cursor.value);
                    cursor.continue();
                } else {
                    console.log("All data retrieved and sorted by id descending");
                    resolve(results);
                }
            };

            request.onerror = (event) => {
                console.error("Error retrieving all data:", event.target.error);
                reject(event.target.error);
            };
        });
    }

    //获取dm task列表
    getDmtasklist(){
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['dm_task'], "readonly");
            const store = transaction.objectStore('dm_task');

            const request = store.openCursor(null, "prev"); // "prev" 表示降序

            const results = [];
            request.onsuccess = (event) => {
                const cursor = event.target.result;
                if (cursor) {
                    results.push(cursor.value);
                    cursor.continue();
                } else {
                    console.log("All data retrieved and sorted by id descending");
                    resolve(results);
                }
            };

            request.onerror = (event) => {
                console.error("Error retrieving all data:", event.target.error);
                reject(event.target.error);
            };
        });
    }

    //dm task item 删除
    dmTaskitemRemove(task_id){
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['dm_task_item'], "readwrite");
            const store = transaction.objectStore('dm_task_item');
            const index = store.index('_task_id_index');  // 获取索引

            const cursorRequest = index.openCursor(IDBKeyRange.only(task_id));

            cursorRequest.onsuccess = (event) => {
                const cursor = event.target.result;

                if (cursor) {
                    // 如果找到符合条件的记录，删除它
                    console.log('cursor.primaryKey:'+cursor.primaryKey); // 获取主键
                    store.delete(cursor.primaryKey);
                    console.log(`Deleted item with primaryKey: ${cursor.primaryKey}`);
                    cursor.continue();  // 继续遍历下一条记录
                } else {
                    console.log("All matching records deleted.");
                    resolve(true);
                }
            };

            cursorRequest.onerror = (event) => {
                console.error("Error during cursor iteration:", event.target.error);
                reject(event.target.error);
            };
        });
    }
    //dm task 删除
    async dmTaskRemove(id){
        let dmtaskitemdelete = await this.dmTaskitemRemove(id);
        if(dmtaskitemdelete){
            this.deleteData('dm_task', id);
        }
        return true;
    }
    //dm template 删除
    async dmTemplateRemove(id){
        await this.deleteData('dm_template', id);
        return true;
    }
    //dm filter 删除
    async dmFilterRemove(id){
        await this.deleteData('dm_filter', id);
        return true;
    }
    //
    async updateDmtaskTotalNum(id){
        let total_num = await this.getDmTaskItemCount(id);
        this.updateDataById('dm_task',id,{'total_num':total_num});
    }

    async updateDmtaskHassuccessnum(id){
        let dmtask = await this.getData('dm_task', id);
        if(dmtask){
            let has_success_num = 0;
            if(dmtask.hasOwnProperty('has_success_num')){
                has_success_num = dmtask['has_success_num'];
            }
            has_success_num = parseInt(has_success_num);
            has_success_num += 1;
            //
            this.updateDataById('dm_task', id, {'has_success_num':has_success_num});
            //
        }
    }

    async updateDmtaskHasfailednum(id){
        let dmtask = await this.getData('dm_task', id);
        if(dmtask){
            let has_failed_num = 0;
            if(dmtask.hasOwnProperty('has_failed_num')){
                has_failed_num = dmtask['has_failed_num'];
            }
            has_failed_num = parseInt(has_failed_num);
            has_failed_num += 1;
            //
            this.updateDataById('dm_task', id, {'has_failed_num':has_failed_num});
            //
        }
    }

    //获取taskItem总行数 已task_id查询
    getDmTaskItemCount(task_id){
        return new Promise((resolve, reject) => {
            let transaction = this.db.transaction(['dm_task_item'], 'readonly');
            let store = transaction.objectStore('dm_task_item');
            const index = store.index('_task_id_index'); // 使用 task_id 索引
            const range = IDBKeyRange.only(task_id); // 查询 task_id = 1
            let countRequest = index.count(range);

            countRequest.onsuccess = () => {
              resolve(countRequest.result);
            };

            countRequest.onerror = () => {
              reject(countRequest.error);
            };
        });
    }

    //只有process和pause两个状态更换
    updateDmTaskStatus(id) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['dm_task'], 'readwrite');
            const store = transaction.objectStore('dm_task');
            // 获取指定 id 的记录
            const getRequest = store.get(id);
            getRequest.onsuccess = (event) => {
                const record = event.target.result;
                if (record) {
                    if(record.status == 'process' || record.status == 'pending'){
                        record.status = 'pause';
                    }else if(record.status == 'pause'){
                        record.status = 'process';
                    }
                    const updateRequest = store.put(record); // 写入更新

                    updateRequest.onsuccess = () => resolve(record.status);
                    updateRequest.onerror = (event) => reject(event.target.error);
                } else {
                    reject(new Error(`Record with id ${id} not found`));
                }
            };

            getRequest.onerror = (event) => reject(event.target.error);
        });
    }

    getDmTaskitemPaginatedData(req_args, page, pageSize = 10){
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(['dm_task_item'], "readonly");
            const store = transaction.objectStore('dm_task_item');
            //
            let _task_id = req_args['_task_id'];
            _task_id = parseInt(_task_id);
            //
            console.log('getDmTaskitemPaginatedData _task_id');
            console.log(_task_id);
            //
            let index = store.index('_task_id_index');
            let keyRange = IDBKeyRange.only(_task_id); // _is_request_index = 0
            const cursorRequest = index.openCursor(keyRange, "next"); // 按 id 升序
            /////
            let taskitems = [];
            cursorRequest.onsuccess = (event) => {
                const cursor = event.target.result;
                if (cursor) {
                    taskitems.push(cursor.value);
                    cursor.continue(); // 继续读取下一条
                } else {
                    // 所有记录已经读取完成，开始分页
                    const start = (page - 1) * pageSize;
                    const end = start + pageSize;
                    const paginatedData = taskitems.slice(start, end);
                    resolve({
                        'total':taskitems.length,
                        'data':paginatedData
                    });
                }
            };
            cursorRequest.onerror = (event) => reject(event.target.error);
            //
        });
    }


    /*
    * 判断dmtask是否完成
    * */
    async dmTaskIsComplete(task_id){
        let pendingNum = await this.getDmtaskitemPendingNum(task_id);
        if(pendingNum == 0){
            return true;
        }
        return false;
    }

    /**
     * 查询需要发送的任务
     */
    findNeedDmTask() {
      return new Promise((resolve, reject) => {
          let transaction = this.db.transaction(['dm_task'], 'readonly');
          let store = transaction.objectStore('dm_task');
          let index = store.index('_status_index');
          let keyRange = IDBKeyRange.only('process');
          let results = [];

          index.openCursor(keyRange).onsuccess = (event) => {
            const cursor = event.target.result;
            if (cursor) {
              const record = cursor.value;
              results.push(record);
              cursor.continue();
            } else {
                // 查询完后，按主键 id 排序
                if(results.length > 0){
                    results.sort((a, b) => a.id - b.id); // 升序
                    resolve(results[0]); // 遍历结束，返回结果
                }else{
                    resolve(null);
                }
            }
          };

          index.openCursor(keyRange).onerror = (event) => {
            reject("游标查询失败: " + event.target.error);
          };

      });
    }

    getDmtaskitemNeedSend(task_id) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(["dm_task_item"], "readonly");
            const store = transaction.objectStore("dm_task_item");
            //
            const index = store.index("_task_id_index");
            const request = index.openCursor(IDBKeyRange.only(task_id));
            //
            let results = [];

            request.onsuccess = (event) => {
              const cursor = event.target.result;
              if (cursor) {
                  const record_data = cursor.value;
                  if(record_data['status'] =='pending'){
                      results.push(record_data);
                  }
                  cursor.continue();
              } else {
                 // 查询完后，按主键 id 排序
                  if(results.length > 0){
                      results.sort((a, b) => a.id - b.id); // 升序
                      //
                      let result = results[0];
                      const now = Date.now();
                      if(now >= result['_s_time']){
                          resolve(results[0]); // 遍历结束，返回结果
                      }
                      resolve(null);
                  }else{
                    resolve(null);
                  }
              }
            };
            request.onerror = () => reject("Error reading task");
        });
    }

    getDmtaskitemPendingNum(task_id) {
        return new Promise((resolve, reject) => {
            const transaction = this.db.transaction(["dm_task_item"], "readonly");
            const store = transaction.objectStore("dm_task_item");
            //
            const index = store.index("_task_id_index");
            const request = index.openCursor(IDBKeyRange.only(task_id));
            //
            let results = [];

            request.onsuccess = (event) => {
              const cursor = event.target.result;
              if (cursor) {
                  const record_data = cursor.value;
                  if(record_data['status'] =='pending'){
                      results.push(record_data);
                  }
                  cursor.continue();
              }else{
                  resolve(results.length);
              }
            };
            request.onerror = () => reject("Error reading task");
        });
    }

    markDmTaskItemAsFailed(dm_task_item_id ,_error) {
        const transaction = this.db.transaction(["dm_task_item"], "readwrite");
        const store = transaction.objectStore("dm_task_item");
        const request = store.get(dm_task_item_id);
        request.onsuccess = (event) => {
            //
            const record = event.target.result;
            if (record) {
                console.log('markTaskAsFailed aa');
                record.status = 'failed';
                record._error = _error;
                store.put(record);
            }
            //
        };
    }

    markDmTaskItemAsSent(dm_task_item_id) {
        const transaction = this.db.transaction(["dm_task_item"], "readwrite");
        const store = transaction.objectStore("dm_task_item");
        const request = store.get(dm_task_item_id);
        request.onsuccess = (event) => {
            //
            const record = event.target.result;
            if (record) {
                record.status = 'complete';
                store.put(record);
            }
            //
        };
    }

    dmTaskItemUpdate(dm_task_item_id, send_res) {
        const transaction = this.db.transaction(["dm_task_item"], "readwrite");
        const store = transaction.objectStore("dm_task_item");
        const request = store.get(dm_task_item_id);
        request.onsuccess = (event) => {
            //
            console.log('sendDM dmTaskItemUpdate');
            const record = event.target.result;
            if (record) {
                record.content = send_res['content'];
                record.full_name = send_res['full_name'];
                record._uid = send_res['_uid'];
                record.avatar = send_res['avatar'];
                store.put(record);
            }
            //
        };
    }

    //更新dmtask pause_hassend_num
    async dmTaskPauseHassend_numUpdate(task_id){
        let dmtask = await this.getData('dm_task', task_id);
        if(!dmtask) return;
        //每发送20消息，暂停20~25分钟
        const dmtask_pause_send_num = dmtask['pause_send_num'];
        let dmtask_pause_hassend_num = dmtask['pause_hassend_num'];
        if(dmtask_pause_hassend_num < dmtask_pause_send_num){
            dmtask_pause_hassend_num += 1;
        }else{
            dmtask_pause_hassend_num = 1;
        }
        //
        console.log('sendDM dmTaskPauseHassend_numUpdate');
        await this.updateDataById('dm_task', task_id, {'pause_hassend_num':dmtask_pause_hassend_num});
        //
    }

    //更新下一封消息发送时间
    async dmTaskItemNextSendTimeUpdate(task_id){
        function util_getRandomDelay(arr = []){
            if(arr.length > 0){
                let delay_minutes = arr[Math.floor(Math.random() * arr.length)];
                return delay_minutes * 60 * 1000; // 转换为毫秒
            }
            return 0;
        }

        try{
            //获取下次需要发送的task item
            let dmtaskitemneed = await this.getDmtaskitemNeedSend(task_id);
            if(!dmtaskitemneed){return;}

            let dmtask = await this.getData('dm_task', task_id);

            //每发送20消息，暂停20~25分钟
            const dmtask_pause_send_num = dmtask['pause_send_num'];
            const dmtask_pause_hassend_num = dmtask['pause_hassend_num'];
            const dmtask_pause_jiange = dmtask['pause_jiange'];
            /*
            * dmtask_pause_jiange = {s:20,e:25}
            * */
            //每条消息间隔分钟
            const dmtask_send_jiange = dmtask['send_jiange'];
            /**
             * dmtask_send_jiange = {s:8,e:15}
             */
            const now = Date.now();
            let delay = util_getRandomDelay([dmtask_send_jiange['s'], dmtask_send_jiange['e']]);
            if(dmtask_pause_hassend_num == dmtask_pause_send_num){
                delay = util_getRandomDelay(dmtask_pause_jiange['s'], dmtask_pause_jiange['e']);
            }
            //
            delay = now + delay;
            //
            let dmtaskitemneedUpdateFields = {
                '_s_time':delay
            };
            await this.updateDataById('dm_task_item', dmtaskitemneed['id'], dmtaskitemneedUpdateFields);
            //

            //
        }catch (err){
            console.error("Update error:", err);
            throw err;
        }
    }
    //

    async markDmTaskComplete(task_id) {
        this.dmTaskIsComplete(task_id)
            .then((res) => {
                if(res){
                    const transaction = this.db.transaction(["dm_task"], "readwrite");
                    const store = transaction.objectStore("dm_task");
                    const request = store.get(task_id);
                    request.onsuccess = (event) => {
                        //
                        const record = event.target.result;
                        if (record) {
                            record.status = 'complete';
                            store.put(record);
                        }
                        //
                    };
                }
            })
            .catch((error) => {

            });
    }
}